@@ -964,8 +964,12 @@ install_theme (GtkWidget *widget, gchar **uris, GtkBuilder *builder)
 
     /* inform the user we are installing the theme */
     gdkwindow = gtk_widget_get_window (widget);
-    cursor = gdk_cursor_new_for_display (gtk_widget_get_display (widget), GDK_WATCH);
-    gdk_window_set_cursor (gdkwindow, cursor);
+    cursor = gdk_cursor_new_from_name (gtk_widget_get_display (widget), "wait");
+    if (cursor != NULL)
+    {
+        gdk_window_set_cursor (gdkwindow, cursor);
+        g_object_unref (cursor);
+    }
 
     /* iterate main loop to show cursor */
     while (gtk_events_pending ())
@@ -1031,7 +1035,6 @@ install_theme (GtkWidget *widget, gchar **uris, GtkBuilder *builder)
 
     g_strfreev (uris);
     gdk_window_set_cursor (gdkwindow, NULL);
-    g_object_unref (cursor);
 
     if (something_installed)
     {
@@ -1445,7 +1448,7 @@ main (gint argc, gchar **argv)
     if (G_UNLIKELY (opt_version))
     {
         g_print ("%s %s (Xfce %s)\n\n", G_LOG_DOMAIN, PACKAGE_VERSION, xfce_version_string ());
-        g_print ("%s\n", "Copyright (c) 2008-2023");
+        g_print ("%s\n", "Copyright (c) 2008-2024");
         g_print ("\t%s\n\n", _("The Xfce development team. All rights reserved."));
         g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
         g_print ("\n");
