@@ -20,6 +20,10 @@
 #include <config.h>
 #endif
 
+#ifdef HAVE_MATH_H
+#include <math.h>
+#endif
+
 #include <gdk/gdkwayland.h>
 #include <libxfce4util/libxfce4util.h>
 
@@ -34,7 +38,6 @@ static void             xfce_display_settings_wayland_finalize
 static guint            xfce_display_settings_wayland_get_n_outputs              (XfceDisplaySettings      *settings);
 static guint            xfce_display_settings_wayland_get_n_active_outputs       (XfceDisplaySettings      *settings);
 static gchar          **xfce_display_settings_wayland_get_display_infos          (XfceDisplaySettings      *settings);
-static MirroredState    xfce_display_settings_wayland_get_mirrored_state         (XfceDisplaySettings      *settings);
 static GdkMonitor      *xfce_display_settings_wayland_get_monitor                (XfceDisplaySettings      *settings,
                                                                                   guint                     output_id);
 static const gchar     *xfce_display_settings_wayland_get_friendly_name          (XfceDisplaySettings      *settings,
@@ -49,16 +52,11 @@ static void             xfce_display_settings_wayland_set_rotation
                                                                                   RotationFlags             rotation);
 static RotationFlags    xfce_display_settings_wayland_get_rotations              (XfceDisplaySettings      *settings,
                                                                                   guint                     output_id);
-static gdouble          xfce_display_settings_wayland_get_scale_x                (XfceDisplaySettings      *settings,
+static gdouble          xfce_display_settings_wayland_get_scale                  (XfceDisplaySettings      *settings,
                                                                                   guint                     output_id);
-static void             xfce_display_settings_wayland_set_scale_x                (XfceDisplaySettings      *settings,
+static void             xfce_display_settings_wayland_set_scale                  (XfceDisplaySettings      *settings,
                                                                                   guint                     output_id,
-                                                                                  gdouble                   scale_x);
-static gdouble          xfce_display_settings_wayland_get_scale_y                (XfceDisplaySettings      *settings,
-                                                                                  guint                     output_id);
-static void             xfce_display_settings_wayland_set_scale_y                (XfceDisplaySettings      *settings,
-                                                                                  guint                     output_id,
-                                                                                  gdouble                   scale_y);
+                                                                                  gdouble                   scale);
 static void             xfce_display_settings_wayland_set_mode                   (XfceDisplaySettings      *settings,
                                                                                   guint                     output_id,
                                                                                   guint                     mode_id);
@@ -84,22 +82,17 @@ static gboolean         xfce_display_settings_wayland_is_primary
 static void             xfce_display_settings_wayland_set_primary                (XfceDisplaySettings      *settings,
                                                                                   guint                     output_id,
                                                                                   gboolean                  primary);
-static gboolean         xfce_display_settings_wayland_is_mirrored                (XfceDisplaySettings      *settings,
-                                                                                  guint                     output_id);
-static gboolean         xfce_display_settings_wayland_is_extended                (XfceDisplaySettings      *settings,
-                                                                                  guint                     output_id_1,
-                                                                                  guint                     output_id_2);
 static gboolean         xfce_display_settings_wayland_is_clonable                (XfceDisplaySettings      *settings);
 static void             xfce_display_settings_wayland_save                       (XfceDisplaySettings      *settings,
-                                                                                  guint                     output_id,
                                                                                   const gchar              *scheme);
 static void             xfce_display_settings_wayland_mirror                     (XfceDisplaySettings      *settings);
 static void             xfce_display_settings_wayland_unmirror                   (XfceDisplaySettings      *settings);
 static void             xfce_display_settings_wayland_update_output_mirror       (XfceDisplaySettings      *settings,
                                                                                   XfceOutput               *output);
-static void             xfce_display_settings_wayland_extend_right               (XfceDisplaySettings      *settings,
+static void             xfce_display_settings_wayland_extend                     (XfceDisplaySettings      *settings,
                                                                                   guint                     output_id_1,
-                                                                                  guint                     output_id_2);
+                                                                                  guint                     output_id_2,
+                                                                                  ExtendedMode              mode);
 
 
 
@@ -129,17 +122,14 @@ xfce_display_settings_wayland_class_init (XfceDisplaySettingsWaylandClass *klass
     settings_class->get_n_outputs = xfce_display_settings_wayland_get_n_outputs;
     settings_class->get_n_active_outputs = xfce_display_settings_wayland_get_n_active_outputs;
     settings_class->get_display_infos = xfce_display_settings_wayland_get_display_infos;
-    settings_class->get_mirrored_state = xfce_display_settings_wayland_get_mirrored_state;
     settings_class->get_monitor = xfce_display_settings_wayland_get_monitor;
     settings_class->get_friendly_name = xfce_display_settings_wayland_get_friendly_name;
     settings_class->get_geometry = xfce_display_settings_wayland_get_geometry;
     settings_class->get_rotation = xfce_display_settings_wayland_get_rotation;
     settings_class->set_rotation = xfce_display_settings_wayland_set_rotation;
     settings_class->get_rotations = xfce_display_settings_wayland_get_rotations;
-    settings_class->get_scale_x = xfce_display_settings_wayland_get_scale_x;
-    settings_class->set_scale_x = xfce_display_settings_wayland_set_scale_x;
-    settings_class->get_scale_y = xfce_display_settings_wayland_get_scale_y;
-    settings_class->set_scale_y = xfce_display_settings_wayland_set_scale_y;
+    settings_class->get_scale = xfce_display_settings_wayland_get_scale;
+    settings_class->set_scale = xfce_display_settings_wayland_set_scale;
     settings_class->set_mode = xfce_display_settings_wayland_set_mode;
     settings_class->update_output_mode = xfce_display_settings_wayland_update_output_mode;
     settings_class->set_position = xfce_display_settings_wayland_set_position;
@@ -149,14 +139,12 @@ xfce_display_settings_wayland_class_init (XfceDisplaySettingsWaylandClass *klass
     settings_class->update_output_active = xfce_display_settings_wayland_update_output_active;
     settings_class->is_primary = xfce_display_settings_wayland_is_primary;
     settings_class->set_primary = xfce_display_settings_wayland_set_primary;
-    settings_class->is_mirrored = xfce_display_settings_wayland_is_mirrored;
-    settings_class->is_extended = xfce_display_settings_wayland_is_extended;
     settings_class->is_clonable = xfce_display_settings_wayland_is_clonable;
     settings_class->save = xfce_display_settings_wayland_save;
     settings_class->mirror = xfce_display_settings_wayland_mirror;
     settings_class->unmirror = xfce_display_settings_wayland_unmirror;
     settings_class->update_output_mirror = xfce_display_settings_wayland_update_output_mirror;
-    settings_class->extend_right = xfce_display_settings_wayland_extend_right;
+    settings_class->extend = xfce_display_settings_wayland_extend;
 }
 
 
@@ -268,37 +256,25 @@ get_clonable_modes (GPtrArray *outputs)
 
 
 
-static MirroredState
-xfce_display_settings_wayland_get_mirrored_state (XfceDisplaySettings *settings)
+static void
+get_geometry_mm (XfceWlrOutput *output,
+                 GdkRectangle *geometry)
 {
-    GPtrArray *outputs;
-    gboolean cloned = TRUE;
-    gboolean mirrored = TRUE;
-    XfceWlrMode **clonable_modes;
-
-    if (!xfce_display_settings_wayland_is_clonable (settings))
-        return MIRRORED_STATE_NONE;
-
-    /* check if mirror settings are on */
-    outputs = xfce_wlr_output_manager_get_outputs (XFCE_DISPLAY_SETTINGS_WAYLAND (settings)->manager);
-    clonable_modes = get_clonable_modes (outputs);
-    for (guint n = 0; n < outputs->len; n++)
+    switch (output->transform)
     {
-        XfceWlrOutput *output = g_ptr_array_index (outputs, n);
-        if (!output->enabled)
-            continue;
+        case XFCE_WLR_TRANSFORM_90:
+        case XFCE_WLR_TRANSFORM_270:
+        case XFCE_WLR_TRANSFORM_FLIPPED_90:
+        case XFCE_WLR_TRANSFORM_FLIPPED_270:
+            geometry->width = output->height;
+            geometry->height = output->width;
+            break;
 
-        mirrored &= xfce_display_settings_wayland_is_mirrored (settings, n);
-        cloned &= mirrored && output->wl_mode == clonable_modes[n]->wl_mode;
-        if (!mirrored)
+        default:
+            geometry->width = output->width;
+            geometry->height = output->height;
             break;
     }
-    g_free (clonable_modes);
-
-    if (cloned == TRUE)
-        return MIRRORED_STATE_CLONED;
-
-    return mirrored ? MIRRORED_STATE_MIRRORED : MIRRORED_STATE_NONE;
 }
 
 
@@ -311,16 +287,18 @@ xfce_display_settings_wayland_get_monitor (XfceDisplaySettings *settings,
     XfceWlrOutput *output = g_ptr_array_index (outputs, output_id);
     GdkDisplay *display = gdk_display_get_default ();
     gint n_monitors = gdk_display_get_n_monitors (display);
+    GdkRectangle geom;
 
     /* try to find the right monitor based on info that the protocol ensures is the same
      * as wl-output, i.e. what GTK uses */
+    get_geometry_mm (output, &geom);
     for (gint n = 0; n < n_monitors; n++)
     {
         GdkMonitor *monitor = gdk_display_get_monitor (display, n);
         if (g_strcmp0 (gdk_monitor_get_model (monitor), output->model) == 0
             && g_strcmp0 (gdk_monitor_get_manufacturer (monitor), output->manufacturer) == 0
-            && gdk_monitor_get_width_mm (monitor) == output->width
-            && gdk_monitor_get_height_mm (monitor) == output->height)
+            && gdk_monitor_get_width_mm (monitor) == geom.width
+            && gdk_monitor_get_height_mm (monitor) == geom.height)
         {
             return monitor;
         }
@@ -401,6 +379,9 @@ get_nth_mode (XfceDisplaySettingsWayland *settings,
               XfceWlrOutput *output,
               guint mode_id)
 {
+    if (mode_id == -1U)
+        return get_preferred_mode (settings, output);
+
     GList *lp = g_list_nth (output->modes, mode_id);
     if (lp != NULL)
         return lp->data;
@@ -419,6 +400,7 @@ xfce_display_settings_wayland_get_geometry (XfceDisplaySettings *settings,
     GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (wsettings->manager);
     XfceWlrOutput *output = g_ptr_array_index (outputs, output_id);
     XfceWlrMode *mode = get_current_mode (wsettings, output);
+    gdouble scale = wl_fixed_to_double (output->scale);
 
     geometry->x = output->x;
     geometry->y = output->y;
@@ -428,13 +410,13 @@ xfce_display_settings_wayland_get_geometry (XfceDisplaySettings *settings,
         case XFCE_WLR_TRANSFORM_270:
         case XFCE_WLR_TRANSFORM_FLIPPED_90:
         case XFCE_WLR_TRANSFORM_FLIPPED_270:
-            geometry->width = mode->height;
-            geometry->height = mode->width;
+            geometry->width = round (mode->height / scale);
+            geometry->height = round (mode->width / scale);
             break;
 
         default:
-            geometry->width = mode->width;
-            geometry->height = mode->height;
+            geometry->width = round (mode->width / scale);
+            geometry->height = round (mode->height / scale);
             break;
     }
 }
@@ -537,8 +519,8 @@ xfce_display_settings_wayland_get_rotations (XfceDisplaySettings *settings,
 
 
 static gdouble
-xfce_display_settings_wayland_get_scale_x (XfceDisplaySettings *settings,
-                                           guint output_id)
+xfce_display_settings_wayland_get_scale (XfceDisplaySettings *settings,
+                                         guint output_id)
 {
     GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (XFCE_DISPLAY_SETTINGS_WAYLAND (settings)->manager);
     XfceWlrOutput *output = g_ptr_array_index (outputs, output_id);
@@ -548,38 +530,13 @@ xfce_display_settings_wayland_get_scale_x (XfceDisplaySettings *settings,
 
 
 static void
-xfce_display_settings_wayland_set_scale_x (XfceDisplaySettings *settings,
-                                           guint output_id,
-                                           gdouble scale_x)
+xfce_display_settings_wayland_set_scale (XfceDisplaySettings *settings,
+                                         guint output_id,
+                                         gdouble scale)
 {
     GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (XFCE_DISPLAY_SETTINGS_WAYLAND (settings)->manager);
     XfceWlrOutput *output = g_ptr_array_index (outputs, output_id);
-    output->scale = wl_fixed_from_double (scale_x);
-}
-
-
-
-static gdouble
-xfce_display_settings_wayland_get_scale_y (XfceDisplaySettings *settings,
-                                           guint output_id)
-{
-    GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (XFCE_DISPLAY_SETTINGS_WAYLAND (settings)->manager);
-    XfceWlrOutput *output = g_ptr_array_index (outputs, output_id);
-    return wl_fixed_to_double (output->scale);
-}
-
-
-
-static void
-xfce_display_settings_wayland_set_scale_y (XfceDisplaySettings *settings,
-                                           guint output_id,
-                                           gdouble scale_y)
-{
-    GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (XFCE_DISPLAY_SETTINGS_WAYLAND (settings)->manager);
-    XfceWlrOutput *output = g_ptr_array_index (outputs, output_id);
-    if (wl_fixed_to_double (output->scale) != scale_y)
-        g_warning ("Scale y (%f) != Scale x (%f): ignored (only one scale is actually supported)",
-                   scale_y, wl_fixed_to_double (output->scale));
+    output->scale = wl_fixed_from_double (scale);
 }
 
 
@@ -599,10 +556,10 @@ xfce_display_settings_wayland_set_mode (XfceDisplaySettings *settings,
 
 
 static void
-output_set_mode (XfceOutput *output,
-                 XfceWlrMode *mode,
-                 guint mode_id,
-                 int32_t transform)
+output_set_mode_and_tranformation (XfceOutput *output,
+                                   XfceWlrMode *mode,
+                                   guint mode_id,
+                                   XfceWlrOutput *xfwl_output)
 {
     if (output->active)
     {
@@ -610,7 +567,8 @@ output_set_mode (XfceOutput *output,
         output->mode->width = mode->width;
         output->mode->height = mode->height;
         output->mode->rate = (gdouble) mode->refresh / 1000;
-        output->rotation = convert_rotation_from_transform (transform);
+        output->rotation = convert_rotation_from_transform (xfwl_output->transform);
+        output->scale = wl_fixed_to_double (xfwl_output->scale);
     }
     else
     {
@@ -619,6 +577,7 @@ output_set_mode (XfceOutput *output,
         output->mode->height = output->pref_height;
         output->mode->rate = 0;
         output->rotation = ROTATION_FLAGS_0;
+        output->scale = 1.0;
     }
 }
 
@@ -633,7 +592,7 @@ xfce_display_settings_wayland_update_output_mode (XfceDisplaySettings *settings,
     GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (wsettings->manager);
     XfceWlrOutput *xfwl_output = g_ptr_array_index (outputs, output->id);
     XfceWlrMode *mode = get_nth_mode (wsettings, xfwl_output, mode_id);
-    output_set_mode (output, mode, mode_id, xfwl_output->transform);
+    output_set_mode_and_tranformation (output, mode, mode_id, xfwl_output);
 }
 
 
@@ -668,10 +627,7 @@ xfce_display_settings_wayland_get_output (XfceDisplaySettings *settings,
 
     output->x = xfwl_output->x;
     output->y = xfwl_output->y;
-    output->scale_x = wl_fixed_to_double (xfwl_output->scale);
-    output->scale_y = wl_fixed_to_double (xfwl_output->scale);
     output->active = xfwl_output->enabled;
-    output->mirrored = xfce_display_settings_wayland_is_mirrored (settings, output_id);
 
     preferred = get_preferred_mode (wsettings, xfwl_output);
     output->pref_width = preferred->width;
@@ -679,7 +635,7 @@ xfce_display_settings_wayland_get_output (XfceDisplaySettings *settings,
 
     xfwl_mode = get_current_mode (wsettings, xfwl_output);
     output->mode = g_new0 (XfceMode, 1);
-    output_set_mode (output, xfwl_mode, g_list_index (xfwl_output->modes, xfwl_mode), xfwl_output->transform);
+    output_set_mode_and_tranformation (output, xfwl_mode, g_list_index (xfwl_output->modes, xfwl_mode), xfwl_output);
 
     output->n_modes = g_list_length (xfwl_output->modes);
     output->modes = g_new (XfceMode *, output->n_modes);
@@ -718,42 +674,6 @@ xfce_display_settings_wayland_set_active (XfceDisplaySettings *settings,
     XfceDisplaySettingsWayland *wsettings = XFCE_DISPLAY_SETTINGS_WAYLAND (settings);
     GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (wsettings->manager);
     XfceWlrOutput *output = g_ptr_array_index (outputs, output_id);
-    if (active)
-    {
-        XfceWlrMode *mode = get_preferred_mode (wsettings, output);
-        output->wl_mode = mode->wl_mode;
-
-        /* position output at bottom right as it was when disabled */
-        output->x = 0;
-        output->y = 0;
-        for (guint n = 0; n < outputs->len; n++)
-        {
-            XfceWlrOutput *output_n = g_ptr_array_index (outputs, n);
-            if (output_n->enabled)
-            {
-                XfceWlrMode *mode_n = get_current_mode (wsettings, output_n);
-                output->x = MAX (output->x, output_n->x + mode_n->width);
-                output->y = MAX (output->y, output_n->y);
-            }
-        }
-    }
-    else
-    {
-        /* realign other outputs if needed */
-        XfceWlrMode *mode = get_current_mode (wsettings, output);
-        for (guint n = 0; n < outputs->len; n++)
-        {
-            XfceWlrOutput *output_n = g_ptr_array_index (outputs, n);
-            if (output_n->enabled && n != output_id)
-            {
-                if (output_n->x > output->x)
-                    output_n->x -= mode->width;
-                if (output_n->y > output->y)
-                    output_n->y -= mode->height;
-            }
-        }
-    }
-
     output->enabled = active;
 }
 
@@ -768,7 +688,7 @@ xfce_display_settings_wayland_update_output_active (XfceDisplaySettings *setting
     GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (wsettings->manager);
     XfceWlrOutput *xfwl_output = g_ptr_array_index (outputs, output->id);
     XfceWlrMode *mode = get_current_mode (wsettings, xfwl_output);
-    output_set_mode (output, mode, g_list_index (xfwl_output->modes, mode), xfwl_output->transform);
+    output_set_mode_and_tranformation (output, mode, g_list_index (xfwl_output->modes, mode), xfwl_output);
 }
 
 
@@ -791,72 +711,25 @@ xfce_display_settings_wayland_set_primary (XfceDisplaySettings *settings,
 
 
 
-static gboolean
-xfce_display_settings_wayland_is_mirrored (XfceDisplaySettings *settings,
-                                           guint output_id)
-{
-    GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (XFCE_DISPLAY_SETTINGS_WAYLAND (settings)->manager);
-    XfceWlrOutput *output = g_ptr_array_index (outputs, output_id);
-    gint x = output->x;
-    gint y = output->y;
-
-    for (guint n = 0; n < outputs->len; n++)
-    {
-        if (n == output_id)
-            continue;
-
-        output = g_ptr_array_index (outputs, n);
-        if (output->x == x && output->y == y)
-            return TRUE;
-    }
-
-    return FALSE;
-}
-
-
-
-static gboolean
-xfce_display_settings_wayland_is_extended (XfceDisplaySettings *settings,
-                                           guint output_id_1,
-                                           guint output_id_2)
-{
-    XfceDisplaySettingsWayland *wsettings = XFCE_DISPLAY_SETTINGS_WAYLAND (settings);
-    GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (wsettings->manager);
-    XfceWlrOutput *output_1 = g_ptr_array_index (outputs, output_id_1);
-    XfceWlrOutput *output_2 = g_ptr_array_index (outputs, output_id_2);
-    XfceWlrMode *mode = get_current_mode (wsettings, output_1);
-    return output_2->x == output_1->x + (gint) mode->width;
-}
-
-
-
 static gboolean
 xfce_display_settings_wayland_is_clonable (XfceDisplaySettings *settings)
 {
-    if (xfce_display_settings_wayland_get_n_active_outputs (settings) > 1)
-    {
-        GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (XFCE_DISPLAY_SETTINGS_WAYLAND (settings)->manager);
-        XfceWlrMode **clonable_modes = get_clonable_modes (outputs);
-        gboolean clonable = clonable_modes != NULL;
-        g_free (clonable_modes);
-        return clonable;
-    }
-
-    return FALSE;
+    GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (XFCE_DISPLAY_SETTINGS_WAYLAND (settings)->manager);
+    XfceWlrMode **clonable_modes = get_clonable_modes (outputs);
+    gboolean clonable = clonable_modes != NULL;
+    g_free (clonable_modes);
+    return clonable;
 }
 
 
 
 static void
-xfce_display_settings_wayland_save (XfceDisplaySettings *settings,
-                                    guint output_id,
-                                    const gchar *scheme)
+output_save (XfceWlrOutput *output,
+             XfceWlrMode *mode,
+             const gchar *friendly_name,
+             XfconfChannel *channel,
+             const gchar *scheme)
 {
-    XfceDisplaySettingsWayland *wsettings = XFCE_DISPLAY_SETTINGS_WAYLAND (settings);
-    GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (wsettings->manager);
-    XfceWlrOutput *output = g_ptr_array_index (outputs, output_id);
-    XfceWlrMode *mode = get_current_mode (wsettings, output);
-    XfconfChannel *channel = xfce_display_settings_get_channel (settings);
     RotationFlags rotation;
     gchar *property, *str_value;
     gdouble scale;
@@ -864,16 +737,16 @@ xfce_display_settings_wayland_save (XfceDisplaySettings *settings,
 
     /* save the device name */
     property = g_strdup_printf ("/%s/%s", scheme, output->name);
-    xfconf_channel_set_string (channel, property, xfce_display_settings_wayland_get_friendly_name (settings, output_id));
+    xfconf_channel_set_string (channel, property, friendly_name);
     g_free (property);
 
     property = g_strdup_printf ("/%s/%s/EDID", scheme, output->name);
-    xfconf_channel_set_string (channel, property, output->serial_number);
+    xfconf_channel_set_string (channel, property, output->edid);
     g_free (property);
 
     /* stop here if output is disabled */
     property = g_strdup_printf ("/%s/%s/Active", scheme, output->name);
-    xfconf_channel_set_bool (channel, property, mode != NULL);
+    xfconf_channel_set_bool (channel, property, output->enabled);
     g_free (property);
     if (!output->enabled)
         return;
@@ -921,10 +794,7 @@ xfce_display_settings_wayland_save (XfceDisplaySettings *settings,
 
     /* save the scale */
     scale = wl_fixed_to_double (output->scale);
-    property = g_strdup_printf ("/%s/%s/Scale/X", scheme, output->name);
-    xfconf_channel_set_double (channel, property, scale);
-    g_free (property);
-    property = g_strdup_printf ("/%s/%s/Scale/Y", scheme, output->name);
+    property = g_strdup_printf ("/%s/%s/Scale", scheme, output->name);
     xfconf_channel_set_double (channel, property, scale);
     g_free (property);
 
@@ -939,21 +809,42 @@ xfce_display_settings_wayland_save (XfceDisplaySettings *settings,
 
 
 
+static void
+xfce_display_settings_wayland_save (XfceDisplaySettings *settings,
+                                    const gchar *scheme)
+{
+    XfceDisplaySettingsWayland *wsettings = XFCE_DISPLAY_SETTINGS_WAYLAND (settings);
+    GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (wsettings->manager);
+    XfconfChannel *channel = xfce_display_settings_get_channel (settings);
+    for (guint n = 0; n < outputs->len; n++)
+    {
+        XfceWlrOutput *output = g_ptr_array_index (outputs, n);
+        XfceWlrMode *mode = get_current_mode (wsettings, output);
+        const gchar *friendly_name = xfce_display_settings_wayland_get_friendly_name (settings, n);
+        output_save (output, mode, friendly_name, channel, scheme);
+    }
+}
+
+
+
 static void
 xfce_display_settings_wayland_mirror (XfceDisplaySettings *settings)
 {
     GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (XFCE_DISPLAY_SETTINGS_WAYLAND (settings)->manager);
     XfceWlrMode **clonable_modes = get_clonable_modes (outputs);
+    if (clonable_modes == NULL)
+    {
+        g_warn_if_reached ();
+        return;
+    }
 
     for (guint n = 0; n < outputs->len; n++)
     {
         XfceWlrOutput *output = g_ptr_array_index (outputs, n);
-        if (!output->enabled)
-            continue;
-
-        if (clonable_modes != NULL)
-            output->wl_mode = clonable_modes[n]->wl_mode;
+        output->enabled = TRUE;
+        output->wl_mode = clonable_modes[n]->wl_mode;
         output->transform = ROTATION_FLAGS_0;
+        output->scale = wl_fixed_from_double (1.0);
         output->x = 0;
         output->y = 0;
     }
@@ -975,6 +866,8 @@ xfce_display_settings_wayland_unmirror (XfceDisplaySettings *settings)
         XfceWlrOutput *output = g_ptr_array_index (outputs, n);
         XfceWlrMode *mode = get_preferred_mode (wsettings, output);
         output->wl_mode = mode->wl_mode;
+        output->transform = ROTATION_FLAGS_0;
+        output->scale = wl_fixed_from_double (1.0);
         output->x = x;
         output->y = 0;
         x += mode->width;
@@ -993,26 +886,47 @@ xfce_display_settings_wayland_update_output_mirror (XfceDisplaySettings *setting
     XfceWlrMode *mode = get_current_mode (wsettings, xfwl_output);
     output->x = xfwl_output->x;
     output->y = xfwl_output->y;
-    output->mirrored = xfce_display_settings_wayland_is_mirrored (settings, output->id);
-    output_set_mode (output, mode, g_list_index (xfwl_output->modes, mode), xfwl_output->transform);
+    output->active = xfwl_output->enabled;
+    output_set_mode_and_tranformation (output, mode, g_list_index (xfwl_output->modes, mode), xfwl_output);
 }
 
 
 
 static void
-xfce_display_settings_wayland_extend_right (XfceDisplaySettings *settings,
-                                            guint output_id_1,
-                                            guint output_id_2)
+xfce_display_settings_wayland_extend (XfceDisplaySettings *settings,
+                                      guint output_id_1,
+                                      guint output_id_2,
+                                      ExtendedMode mode)
 {
     XfceDisplaySettingsWayland *wsettings = XFCE_DISPLAY_SETTINGS_WAYLAND (settings);
     GPtrArray *outputs = xfce_wlr_output_manager_get_outputs (wsettings->manager);
     XfceWlrOutput *output_1 = g_ptr_array_index (outputs, output_id_1);
     XfceWlrOutput *output_2 = g_ptr_array_index (outputs, output_id_2);
-    XfceWlrMode *mode = get_current_mode (wsettings, output_1);
+    GdkRectangle geom_1, geom_2;
+
+    xfce_display_settings_wayland_get_geometry (settings, output_id_1, &geom_1);
+    xfce_display_settings_wayland_get_geometry (settings, output_id_2, &geom_2);
     output_1->x = 0;
     output_1->y = 0;
-    output_2->x = mode->width;
+    output_2->x = 0;
     output_2->y = 0;
+    switch (mode)
+    {
+        case EXTENDED_MODE_RIGHT:
+            output_2->x = geom_1.width;
+            break;
+        case EXTENDED_MODE_LEFT:
+            output_1->x = geom_2.width;
+            break;
+        case EXTENDED_MODE_UP:
+            output_1->y = geom_2.height;
+            break;
+        case EXTENDED_MODE_DOWN:
+            output_2->y = geom_1.height;
+            break;
+        default:
+            break;
+    }
 }
 
 
@@ -1046,14 +960,8 @@ manager_listener (XfceWlrOutputManager *manager,
             g_warning ("Output '%s' has no modes, disabling it", output->name);
         }
     }
-    xfce_display_settings_set_outputs (settings);
 
-    xfce_display_settings_populate_combobox (settings);
-    xfce_display_settings_populate_profile_list (settings);
-    xfce_display_settings_populate_popups (settings);
-    xfce_display_settings_set_popups_visible (settings, FALSE);
-
-    foo_scroll_area_invalidate (FOO_SCROLL_AREA (xfce_display_settings_get_scroll_area (settings)));
+    xfce_display_settings_reload (settings);
 }
 
 
