@@ -99,10 +99,7 @@ struct _XfceDisplaysHelperX11
 {
     XfceDisplaysHelper  __parent__;
 
-#ifdef HAS_RANDR_ONE_POINT_THREE
-    gint                has_1_3;
     gint                primary;
-#endif
 
     GdkDisplay         *display;
     GdkWindow          *root_window;
@@ -110,6 +107,7 @@ struct _XfceDisplaysHelperX11
     gint                event_base;
 
     /* RandR cache */
+    XfceRandr          *randr;
     XRRScreenResources *resources;
     GPtrArray          *crtcs;
     GPtrArray          *outputs;
@@ -177,7 +175,6 @@ xfce_displays_helper_x11_class_init (XfceDisplaysHelperX11Class *klass)
 static void
 xfce_displays_helper_x11_init (XfceDisplaysHelperX11 *helper)
 {
-    gint major = 0, minor = 0;
     gint error_base, err;
 
     helper->resources = NULL;
@@ -192,48 +189,38 @@ xfce_displays_helper_x11_init (XfceDisplaysHelperX11 *helper)
     /* check if the randr extension is running */
     if (XRRQueryExtension (helper->xdisplay, &helper->event_base, &error_base))
     {
-        /* query the version */
-        if (XRRQueryVersion (helper->xdisplay, &major, &minor)
-            && (major > 1 || (major == 1 && minor >= 2)))
+        GError *error = NULL;
+        helper->randr = xfce_randr_new (helper->display, &error);
+        if (helper->randr == NULL)
+            g_critical ("%s", error->message);
+
+        gdk_x11_display_error_trap_push (helper->display);
+        /* get the screen resource */
+        helper->resources = XRRGetScreenResources (helper->xdisplay,
+                                                   GDK_WINDOW_XID (helper->root_window));
+        gdk_display_flush (helper->display);
+        err = gdk_x11_display_error_trap_pop (helper->display);
+        if (err)
         {
-            gdk_x11_display_error_trap_push (helper->display);
-            /* get the screen resource */
-            helper->resources = XRRGetScreenResources (helper->xdisplay,
-                                                       GDK_WINDOW_XID (helper->root_window));
-            gdk_display_flush (helper->display);
-            err = gdk_x11_display_error_trap_pop (helper->display);
-            if (err)
-            {
-                g_critical ("XRRGetScreenResources failed (err: %d). "
-                            "Display settings won't be applied.", err);
-                return;
-            }
-
-            /* get all existing CRTCs and connected outputs */
-            helper->crtcs = xfce_displays_helper_x11_list_crtcs (helper);
-            helper->outputs = xfce_displays_helper_x11_list_outputs (helper);
-
-            /* Set up RandR notifications */
-            XRRSelectInput (helper->xdisplay,
-                            GDK_WINDOW_XID (helper->root_window),
-                            RRScreenChangeNotifyMask);
-            gdk_x11_register_standard_event_type (helper->display,
-                                                  helper->event_base,
-                                                  RRNotify + 1);
-            gdk_window_add_filter (helper->root_window,
-                                   xfce_displays_helper_x11_screen_on_event,
-                                   helper);
-
-#ifdef HAS_RANDR_ONE_POINT_THREE
-            helper->has_1_3 = (major > 1 || (major == 1 && minor >= 3));
-#endif
-        }
-        else
-        {
-             g_critical ("RANDR extension is too old, version %d.%d. "
-                         "Display settings won't be applied.",
-                         major, minor);
+            g_critical ("XRRGetScreenResources failed (err: %d). "
+                        "Display settings won't be applied.", err);
+            return;
         }
+
+        /* get all existing CRTCs and connected outputs */
+        helper->crtcs = xfce_displays_helper_x11_list_crtcs (helper);
+        helper->outputs = xfce_displays_helper_x11_list_outputs (helper);
+
+        /* Set up RandR notifications */
+        XRRSelectInput (helper->xdisplay,
+                        GDK_WINDOW_XID (helper->root_window),
+                        RRScreenChangeNotifyMask);
+        gdk_x11_register_standard_event_type (helper->display,
+                                              helper->event_base,
+                                              RRNotify + 1);
+        gdk_window_add_filter (helper->root_window,
+                               xfce_displays_helper_x11_screen_on_event,
+                               helper);
     }
     else
     {
@@ -253,6 +240,12 @@ xfce_displays_helper_x11_dispose (GObject *object)
                               xfce_displays_helper_x11_screen_on_event,
                               helper);
 
+    if (helper->randr)
+    {
+        xfce_randr_free (helper->randr);
+        helper->randr = NULL;
+    }
+
     if (helper->outputs)
     {
         g_ptr_array_unref (helper->outputs);
@@ -314,16 +307,25 @@ xfce_displays_helper_x11_toggle_internal (gpointer           *power,
     guint          n;
     gint           m;
 
-    for (n = 0; n < helper->outputs->len; ++n)
+    if (helper->outputs->len == 1)
     {
-        output = g_ptr_array_index (helper->outputs, n);
-        g_assert (output);
-
-        /* Try to find the internal display */
-        if (display_name_is_laptop_name (output->info->name))
+        /* If there's only one output left and we pass here, it's supposed to be the
+         * internal display or an output we want to reactivate anyway */
+        lvds = g_ptr_array_index (helper->outputs, 0);
+    }
+    else
+    {
+        for (n = 0; n < helper->outputs->len; ++n)
         {
-            lvds = output;
-            break;
+            output = g_ptr_array_index (helper->outputs, n);
+            g_assert (output);
+
+            /* Try to find the internal display */
+            if (display_name_is_laptop_name (output->info->name))
+            {
+                lvds = output;
+                break;
+            }
         }
     }
 
@@ -409,26 +411,10 @@ static gchar **
 xfce_displays_helper_x11_get_display_infos (XfceDisplaysHelper *_helper)
 {
     XfceDisplaysHelperX11 *helper = XFCE_DISPLAYS_HELPER_X11 (_helper);
-    gchar    **display_infos;
-    guint      m;
-    guint8    *edid_data;
+    if (helper->randr != NULL)
+        return xfce_randr_get_display_infos (helper->randr);
 
-    display_infos = g_new0 (gchar *, helper->outputs->len + 1);
-    /* get all display edids, to only query randr once */
-    for (m = 0; m < helper->outputs->len; ++m)
-    {
-        XfceRROutput *output;
-
-        output = g_ptr_array_index (helper->outputs, m);
-        edid_data = xfce_randr_read_edid_data (helper->xdisplay, output->id);
-
-        if (edid_data)
-            display_infos[m] = g_compute_checksum_for_data (G_CHECKSUM_SHA1 , edid_data, 128);
-        else
-            display_infos[m] = g_strdup ("");
-    }
-
-    return display_infos;
+    return NULL;
 }
 
 
@@ -441,12 +427,9 @@ xfce_displays_helper_x11_channel_apply (XfceDisplaysHelper *_helper,
     XfconfChannel *channel = xfce_displays_helper_get_channel (_helper);
     gchar       property[512];
     guint       n, nactive;
-    GHashTable *saved_outputs;
+    GHashTable *saved_outputs = NULL;
 
-    saved_outputs = NULL;
-#ifdef HAS_RANDR_ONE_POINT_THREE
     helper->primary = None;
-#endif
 
     xfconf_channel_set_string (channel, ACTIVE_PROFILE, scheme);
 
@@ -494,6 +477,9 @@ xfce_displays_helper_x11_reload (XfceDisplaysHelperX11 *helper)
 
     xfsettings_dbg (XFSD_DEBUG_DISPLAYS, "Refreshing RandR cache.");
 
+    if (helper->randr != NULL)
+        xfce_randr_reload (helper->randr);
+
     /* Free the caches */
     g_ptr_array_unref (helper->outputs);
     g_ptr_array_unref (helper->crtcs);
@@ -504,17 +490,10 @@ xfce_displays_helper_x11_reload (XfceDisplaysHelperX11 *helper)
     XRRFreeScreenResources (helper->resources);
 
     /* get the screen resource */
-#ifdef HAS_RANDR_ONE_POINT_THREE
     /* xfce_displays_helper_x11_reload () is usually called after a xrandr notification,
        which means that X is aware of the new hardware already. So, if possible,
        do not reprobe the hardware again. */
-    if (helper->has_1_3)
-        helper->resources = XRRGetScreenResourcesCurrent (helper->xdisplay,
-                                                          GDK_WINDOW_XID (helper->root_window));
-    else
-#endif
-    helper->resources = XRRGetScreenResources (helper->xdisplay,
-                                               GDK_WINDOW_XID (helper->root_window));
+    helper->resources = XRRGetScreenResourcesCurrent (helper->xdisplay, GDK_WINDOW_XID (helper->root_window));
 
     gdk_display_flush (helper->display);
     err = gdk_x11_display_error_trap_pop (helper->display);
@@ -562,14 +541,20 @@ xfce_displays_helper_x11_screen_on_event (GdkXEvent *xevent,
 
         /* Check if we have different amount of outputs and a matching profile and
            apply it if there's only one */
-        if (old_outputs->len > helper->outputs->len ||
-            old_outputs->len < helper->outputs->len)
+        if (helper->outputs->len != old_outputs->len)
         {
-            const gchar *matching_profile = xfce_displays_helper_get_matching_profile (XFCE_DISPLAYS_HELPER (helper));
-            if (matching_profile != NULL)
+            gint mode = xfconf_channel_get_int (channel, AUTO_ENABLE_PROFILES, AUTO_ENABLE_PROFILES_DEFAULT);
+            if (mode == AUTO_ENABLE_PROFILES_ALWAYS
+                || (mode == AUTO_ENABLE_PROFILES_ON_CONNECT && helper->outputs->len > old_outputs->len)
+                || (mode == AUTO_ENABLE_PROFILES_ON_DISCONNECT && helper->outputs->len < old_outputs->len))
             {
-                xfce_displays_helper_x11_channel_apply (XFCE_DISPLAYS_HELPER (helper), matching_profile);
-                return GDK_FILTER_CONTINUE;
+                gchar *matching_profile = xfce_displays_helper_get_matching_profile (XFCE_DISPLAYS_HELPER (helper));
+                if (matching_profile != NULL)
+                {
+                    xfce_displays_helper_x11_channel_apply (XFCE_DISPLAYS_HELPER (helper), matching_profile);
+                    g_free (matching_profile);
+                    return GDK_FILTER_CONTINUE;
+                }
             }
             xfconf_channel_set_string (channel, ACTIVE_PROFILE, DEFAULT_SCHEME_NAME);
         }
@@ -622,7 +607,7 @@ xfce_displays_helper_x11_screen_on_event (GdkXEvent *xevent,
         }
         else
         {
-            action = xfconf_channel_get_int (channel, NOTIFY_PROP, ACTION_ON_NEW_OUTPUT_SHOW_DIALOG);
+            action = xfconf_channel_get_int (channel, NOTIFY_PROP, ACTION_ON_NEW_OUTPUT_DEFAULT);
 
             /* Diff the new and old output list to find new outputs */
             for (n = 0; n < helper->outputs->len; ++n)
@@ -644,12 +629,12 @@ xfce_displays_helper_x11_screen_on_event (GdkXEvent *xevent,
                         crtc = xfce_displays_helper_x11_find_usable_crtc (helper, output);
                         if (crtc)
                         {
-                            crtc->mode = output->preferred_mode;
+                            crtc->mode = (action != ACTION_ON_NEW_OUTPUT_DO_NOTHING) ? output->preferred_mode : None;
                             crtc->rotation = RR_Rotate_0;
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
                             if ((crtc->x > gdk_screen_width() + 1) || (crtc->y > gdk_screen_height() + 1)
-                                || action == ACTION_ON_NEW_OUTPUT_MIRROR) {
-G_GNUC_END_IGNORE_DEPRECATIONS
+                                || action == ACTION_ON_NEW_OUTPUT_MIRROR)
+                            {
                                 crtc->x = crtc->y = 0;
                             }
                             /* Extend to the right if configured */
@@ -658,6 +643,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
                                 crtc->x = helper->width - crtc->width;
                                 crtc->y = 0;
                             } /* else - leave values from last time we saw the monitor */
+G_GNUC_END_IGNORE_DEPRECATIONS
                             /* set width and height */
                             for (j = 0; j < helper->resources->nmode; ++j)
                             {
@@ -679,9 +665,12 @@ G_GNUC_END_IGNORE_DEPRECATIONS
             if (changed)
                 xfce_displays_helper_x11_apply_all (helper);
 
-            /* Start the minimal dialog according to the user preferences */
+            /* Start the display dialog according to the user preferences */
             if (changed && action == ACTION_ON_NEW_OUTPUT_SHOW_DIALOG)
-                xfce_spawn_command_line (NULL, "xfce4-display-settings -m", FALSE, FALSE, TRUE, NULL);
+            {
+                const gchar *cmd = helper->outputs->len <= 2 ? "xfce4-display-settings -m" : "xfce4-display-settings";
+                xfce_spawn_command_line (NULL, cmd, FALSE, FALSE, TRUE, NULL);
+            }
         }
         g_ptr_array_unref (old_outputs);
     }
@@ -750,7 +739,7 @@ xfce_displays_helper_x11_load_from_xfconf (XfceDisplaysHelperX11 *helper,
     const gchar *str_value;
     gchar        property[512];
     gdouble      output_rate, rate;
-    gdouble      scalex, scaley;
+    gdouble      scale;
     RRMode       valid_mode;
     Rotation     rot;
     gint         x, y, n, m, int_value;
@@ -767,17 +756,12 @@ xfce_displays_helper_x11_load_from_xfconf (XfceDisplaysHelperX11 *helper,
     if (value == NULL || !G_VALUE_HOLDS_STRING (value))
         return active;
 
-#ifdef HAS_RANDR_ONE_POINT_THREE
-    if (helper->has_1_3)
-    {
-        /* is it the primary output? */
-        g_snprintf (property, sizeof (property), PRIMARY_PROP, scheme,
-                    output->info->name);
-        value = g_hash_table_lookup (saved_outputs, property);
-        if (G_VALUE_HOLDS_BOOLEAN (value) && g_value_get_boolean (value))
-            helper->primary = output->id;
-    }
-#endif
+    /* is it the primary output? */
+    g_snprintf (property, sizeof (property), PRIMARY_PROP, scheme,
+                output->info->name);
+    value = g_hash_table_lookup (saved_outputs, property);
+    if (G_VALUE_HOLDS_BOOLEAN (value) && g_value_get_boolean (value))
+        helper->primary = output->id;
 
     /* status */
     g_snprintf (property, sizeof (property), ACTIVE_PROP, scheme, output->info->name);
@@ -870,40 +854,29 @@ xfce_displays_helper_x11_load_from_xfconf (XfceDisplaysHelperX11 *helper,
     else
         output_rate = 0.0;
 
-#ifdef HAS_RANDR_ONE_POINT_THREE
-    if (helper->has_1_3)
+    /* scaling */
+    g_snprintf (property, sizeof (property), SCALE_PROP, scheme, output->info->name);
+    value = g_hash_table_lookup (saved_outputs, property);
+    if (G_VALUE_HOLDS_DOUBLE (value))
+        scale = g_value_get_double (value);
+    else
+        scale = 1.0;
+
+    /* backward compatibility; old properties are reset in xfce-randr.c when saving  */
+    g_snprintf (property, sizeof (property), SCALEX_PROP, scheme, output->info->name);
+    value = g_hash_table_lookup (saved_outputs, property);
+    if (G_VALUE_HOLDS_DOUBLE (value))
+        scale = g_value_get_double (value);
+
+    if (scale <= 0.0)
+        scale = 1.0;
+
+    if (crtc->scalex != scale || crtc->scaley != scale)
     {
-        /* scaling X */
-        g_snprintf (property, sizeof (property), SCALEX_PROP, scheme,
-                    output->info->name);
-        value = g_hash_table_lookup (saved_outputs, property);
-        if (G_VALUE_HOLDS_DOUBLE (value))
-            scalex = g_value_get_double (value);
-        else
-            scalex = 1.0;
-
-        /* scaling Y */
-        g_snprintf (property, sizeof (property), SCALEY_PROP, scheme,
-                    output->info->name);
-        value = g_hash_table_lookup (saved_outputs, property);
-        if (G_VALUE_HOLDS_DOUBLE (value))
-            scaley = g_value_get_double (value);
-        else
-            scaley = 1.0;
-
-        if (scalex <= 0.0 || scaley <= 0.0) {
-            scalex = 1.0;
-            scaley = 1.0;
-        }
-
-        if (crtc->scalex != scalex || crtc->scaley != scaley)
-        {
-            crtc->scalex = scalex;
-            crtc->scaley = scaley;
-            crtc->changed = TRUE;
-        }
+        crtc->scalex = scale;
+        crtc->scaley = scale;
+        crtc->changed = TRUE;
     }
-#endif
 
     /* check mode validity */
     valid_mode = None;
@@ -1296,8 +1269,8 @@ xfce_displays_helper_x11_normalize_crtc (XfceRRCrtc            *crtc,
                     crtc->id, crtc->width, crtc->height, crtc->x, crtc->y);
 
     /* calculate the total screen size */
-    helper->width = MAX (helper->width, crtc->x + crtc->width * crtc->scalex);
-    helper->height = MAX (helper->height, crtc->y + crtc->height * crtc->scaley);
+    helper->width = MAX (helper->width, crtc->x + round (crtc->width * crtc->scalex));
+    helper->height = MAX (helper->height, crtc->y + round (crtc->height * crtc->scaley));
 
     /* The 'physical size' of an X screen is meaningless if that screen
      * can consist of many monitors. So just pick a size that make the
@@ -1305,8 +1278,8 @@ xfce_displays_helper_x11_normalize_crtc (XfceRRCrtc            *crtc,
      *
      * Firefox and Evince apparently believe what X tells them.
      */
-    helper->mm_width = (helper->width / 96.0) * 25.4 + 0.5;
-    helper->mm_height = (helper->height / 96.0) * 25.4 + 0.5;
+    helper->mm_width = round ((helper->width / 96.0) * 25.4 + 0.5);
+    helper->mm_height = round ((helper->height / 96.0) * 25.4 + 0.5);
 }
 
 
@@ -1363,35 +1336,30 @@ xfce_displays_helper_x11_apply_crtc_transform (XfceRRCrtc            *crtc,
     if (!crtc->changed)
         return;
 
-#ifdef HAS_RANDR_ONE_POINT_THREE
-    if (helper->has_1_3)
+    if (crtc->scalex == 1 && crtc->scaley == 1)
+        filter = "nearest";
+    else
+        filter = "bilinear";
+
+    xfsettings_dbg (XFSD_DEBUG_DISPLAYS, "Applying CRTC %lu Transform: x=%lf y=%lf, filter=%s.", crtc->id,
+                    crtc->scalex, crtc->scaley, filter);
+
+    memset(&transform, '\0', sizeof(transform));
+
+    transform.matrix[0][0] = XDoubleToFixed(crtc->scalex);
+    transform.matrix[1][1] = XDoubleToFixed(crtc->scaley);
+    transform.matrix[2][2] = XDoubleToFixed(1.0);
+
+    gdk_x11_display_error_trap_push (helper->display);
+    XRRSetCrtcTransform(helper->xdisplay, crtc->id,
+                        &transform,
+                        filter,
+                        NULL,
+                        0);
+    if (gdk_x11_display_error_trap_pop (helper->display) != 0)
     {
-        if (crtc->scalex == 1 && crtc->scaley == 1)
-            filter = "nearest";
-        else
-            filter = "bilinear";
-
-        xfsettings_dbg (XFSD_DEBUG_DISPLAYS, "Applying CRTC %lu Transform: x=%lf y=%lf, filter=%s.", crtc->id,
-                        crtc->scalex, crtc->scaley, filter);
-
-        memset(&transform, '\0', sizeof(transform));
-
-        transform.matrix[0][0] = XDoubleToFixed(crtc->scalex);
-        transform.matrix[1][1] = XDoubleToFixed(crtc->scaley);
-        transform.matrix[2][2] = XDoubleToFixed(1.0);
-
-        gdk_x11_display_error_trap_push (helper->display);
-        XRRSetCrtcTransform(helper->xdisplay, crtc->id,
-                            &transform,
-                            filter,
-                            NULL,
-                            0);
-        if (gdk_x11_display_error_trap_pop (helper->display) != 0)
-        {
-            g_warning ("Failed to apply the scale, maybe the CRTC does not support transforms");
-        }
+        g_warning ("Failed to apply the scale, maybe the CRTC does not support transforms");
     }
-#endif
 }
 
 
@@ -1492,17 +1460,13 @@ xfce_displays_helper_x11_apply_all (XfceDisplaysHelperX11 *helper)
     /* final loop, apply crtc changes */
     g_ptr_array_foreach (helper->crtcs, (GFunc) xfce_displays_helper_x11_apply_crtc, helper);
 
-#ifdef HAS_RANDR_ONE_POINT_THREE
-        if (helper->has_1_3)
-            XRRSetOutputPrimary (helper->xdisplay, GDK_WINDOW_XID (helper->root_window),
-                                 helper->primary);
-#endif
+    XRRSetOutputPrimary (helper->xdisplay, GDK_WINDOW_XID (helper->root_window), helper->primary);
 
     /* release the grab, changes are done */
     gdk_display_sync (helper->display);
     gdk_x11_display_ungrab (helper->display);
     if (gdk_x11_display_error_trap_pop (helper->display) != 0)
     {
-        g_critical ("Failed to apply display settings");
+        g_warning ("Failed to apply display settings");
     }
 }
