@@ -25,15 +25,12 @@
 
 #include <xfconf/xfconf.h>
 #include <libxfce4util/libxfce4util.h>
-#include <gdk/gdk.h>
 
-#ifdef GDK_WINDOWING_X11
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <gdk/gdkx.h>
-#endif
 
-#include "debug.h"
+#include "common/debug.h"
 #include "workspaces.h"
 
 #define WORKSPACES_CHANNEL    "xfwm4"
@@ -63,12 +60,8 @@ struct _XfceWorkspacesHelper
     GObject parent;
 
     XfconfChannel *channel;
-
-    GTimeVal       timestamp;
-
-#ifdef GDK_WINDOWING_X11
+    gint64         timestamp;
     guint          wait_for_wm_timeout_id;
-#endif
 };
 
 struct _XfceWorkspacesHelperClass
@@ -76,11 +69,9 @@ struct _XfceWorkspacesHelperClass
     GObjectClass parent;
 };
 
-
-
-#ifdef GDK_WINDOWING_X11
 static Atom atom_net_number_of_desktops = 0;
 static Atom atom_net_desktop_names = 0;
+static gboolean xfsettingsd_disable_wm_check = FALSE;
 
 typedef struct
 {
@@ -93,7 +84,6 @@ typedef struct
   guint                 counter;
 }
 WaitForWM;
-#endif
 
 
 
@@ -108,10 +98,8 @@ xfce_workspaces_helper_class_init(XfceWorkspacesHelperClass *klass)
 
     gobject_class->finalize = xfce_workspaces_helper_finalize;
 
-#ifdef GDK_WINDOWING_X11
     atom_net_number_of_desktops = gdk_x11_get_xatom_by_name ("_NET_NUMBER_OF_DESKTOPS");
     atom_net_desktop_names = gdk_x11_get_xatom_by_name ("_NET_DESKTOP_NAMES");
-#endif
 }
 
 
@@ -157,10 +145,8 @@ xfce_workspaces_helper_filter_func (GdkXEvent  *gdkxevent,
                                     GdkEvent   *event,
                                     gpointer    user_data)
 {
-#ifdef GDK_WINDOWING_X11
     XfceWorkspacesHelper  *helper = XFCE_WORKSPACES_HELPER (user_data);
     XEvent                *xevent = gdkxevent;
-    GTimeVal               timestamp;
 
     if (xevent->type == PropertyNotify)
     {
@@ -174,10 +160,7 @@ xfce_workspaces_helper_filter_func (GdkXEvent  *gdkxevent,
         else if (xevent->xproperty.atom == atom_net_desktop_names)
         {
             /* don't respond to our own name changes (1 sec) */
-            g_get_current_time (&timestamp);
-            if (timestamp.tv_sec > helper->timestamp.tv_sec
-                || (timestamp.tv_sec == helper->timestamp.tv_sec
-                    && timestamp.tv_usec > helper->timestamp.tv_usec))
+            if (g_get_real_time () > helper->timestamp)
             {
                 /* someone changed (possibly another application that does
                  * not update xfconf) the name of a desktop, store the
@@ -188,7 +171,6 @@ xfce_workspaces_helper_filter_func (GdkXEvent  *gdkxevent,
             }
         }
     }
-#endif
 
     return GDK_FILTER_CONTINUE;
 }
@@ -343,8 +325,7 @@ xfce_workspaces_helper_set_names_real (XfceWorkspacesHelper *helper)
         }
 
         /* update stamp so new names is not handled for the next second */
-        g_get_current_time (&helper->timestamp);
-        g_time_val_add (&helper->timestamp, G_USEC_PER_SEC);
+        helper->timestamp = g_get_real_time () + G_USEC_PER_SEC;
 
         gdk_x11_display_error_trap_push (gdk_display_get_default ());
 
@@ -412,7 +393,6 @@ xfce_workspaces_helper_set_names_real (XfceWorkspacesHelper *helper)
 
 
 
-#ifdef GDK_WINDOWING_X11
 static gboolean
 xfce_workspaces_helper_wait_for_window_manager (gpointer data)
 {
@@ -463,7 +443,6 @@ xfce_workspaces_helper_wait_for_window_manager_destroyed (gpointer data)
   /* set the names anyway... */
   xfce_workspaces_helper_set_names_real (helper);
 }
-#endif
 
 
 
@@ -471,12 +450,11 @@ static void
 xfce_workspaces_helper_set_names (XfceWorkspacesHelper *helper,
                                   gboolean              disable_wm_check)
 {
-#ifdef GDK_WINDOWING_X11
     WaitForWM  *wfwm;
     guint       i;
     gchar     **atom_names;
 
-    if (!disable_wm_check)
+    if (!disable_wm_check && !xfsettingsd_disable_wm_check)
     {
         /* setup data for wm checking */
         wfwm = g_slice_new0 (WaitForWM);
@@ -504,7 +482,6 @@ xfce_workspaces_helper_set_names (XfceWorkspacesHelper *helper,
                               wfwm, xfce_workspaces_helper_wait_for_window_manager_destroyed);
     }
     else
-#endif
     {
         /* directly launch */
         xfce_workspaces_helper_set_names_real (helper);
@@ -592,3 +569,11 @@ xfce_workspaces_helper_prop_changed (XfconfChannel        *channel,
         xfce_workspaces_helper_set_names (helper, TRUE);
     }
 }
+
+
+
+void
+xfce_workspaces_helper_disable_wm_check (gboolean disable_wm_check)
+{
+    xfsettingsd_disable_wm_check = disable_wm_check;
+}
